﻿Imports System.Threading
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices

Public Class Form1
    Private m_Filter(,) As Integer
    Private m_Weight As Integer

    ' Initialize the filter.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        m_Filter = New Integer(,) { _
            {-1, 0, 0}, _
            {0, 0, 0}, _
            {0, 0, 1}}
        m_Weight = 1
    End Sub

    ' Open an image file.
    Private Sub mnuFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileOpen.Click
        If ofdOld.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Dim bm As New Bitmap(ofdOld.FileName)
            picOld.Image = bm.Clone()
            picNew.Image = bm.Clone()
        End If
    End Sub

#Region "Filter"
    ' Process the image sequentially.
    Private Sub btnFilter_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFilter.Click
        If picOld.Image Is Nothing Then
            MessageBox.Show("You must load an image first", "No Image", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Sub
        End If

        lblTime.Text = ""
        picNew.Image = picOld.Image.Clone()
        Me.Cursor = Cursors.WaitCursor
        Me.Refresh()

        Dim start_time As Date = Now
        DoFilter()
        Dim stop_time As Date = Now
        Dim elapsed_time As TimeSpan = stop_time - start_time

        picNew.Refresh()
        lblTime.Text = elapsed_time.TotalSeconds.ToString("0.00") & " sec"
        Me.Cursor = Cursors.Default
    End Sub

    ' Apply the filter sequentially.
    Private Sub DoFilter()
        Dim bm1 As Bitmap = picOld.Image
        Dim bm2 As Bitmap = picNew.Image

        For x As Integer = 1 To bm1.Width - 2
            For y As Integer = 1 To bm1.Height - 2
                Dim r As Integer = 0
                Dim g As Integer = 0
                Dim b As Integer = 0
                For dx As Integer = 0 To 2
                    For dy As Integer = 0 To 2
                        Dim clr As Color = bm1.GetPixel(x + dx - 1, y + dy - 1)
                        r += clr.R * m_Filter(dx, dy)
                        g += clr.G * m_Filter(dx, dy)
                        b += clr.B * m_Filter(dx, dy)
                    Next dy
                Next dx

                r = CInt(127 + r / m_Weight)
                g = CInt(127 + g / m_Weight)
                b = CInt(127 + b / m_Weight)
                If r < 0 Then r = 0
                If g < 0 Then g = 0
                If b < 0 Then b = 0
                If r > 255 Then r = 255
                If g > 255 Then g = 255
                If b > 255 Then b = 255
                bm2.SetPixel(x, y, Color.FromArgb(r, g, b))
            Next y
        Next x
    End Sub
#End Region ' Filter

#Region "LockBits"
    ' Process the image sequentially using LockBits.
    Private Sub btnLockBits_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLockBits.Click
        If picOld.Image Is Nothing Then
            MessageBox.Show("You must load an image first", "No Image", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Sub
        End If

        lblLockBitsTime.Text = ""
        picNew.Image = picOld.Image.Clone()
        Me.Cursor = Cursors.WaitCursor
        Me.Refresh()

        Dim start_time As Date = Now
        DoFilterLockBits()
        Dim stop_time As Date = Now
        Dim elapsed_time As TimeSpan = stop_time - start_time

        picNew.Refresh()
        lblLockBitsTime.Text = elapsed_time.TotalSeconds.ToString("0.00") & " sec"
        Me.Cursor = Cursors.Default
    End Sub

    ' Apply the filter sequentially.
    Private Sub DoFilterLockBits()
        Dim bm1 As Bitmap = picOld.Image
        Dim bm2 As Bitmap = picNew.Image

        ' Make BitmapBytesRGB24 objects for the bitmaps.
        Dim bm1_bytes As New BitmapBytesRGB24(bm1)
        Dim bm2_bytes As New BitmapBytesRGB24(bm2)

        ' Lock the bitmaps' bytes.
        bm1_bytes.LockBitmap()
        bm2_bytes.LockBitmap()

        Dim wid As Integer = bm1.Width
        Dim hgt As Integer = bm1.Height
        Dim new_r As Byte = 0
        Dim new_g As Byte = 0
        Dim new_b As Byte = 0
        For x As Integer = 1 To wid - 2
            For y As Integer = 1 To hgt - 2
                Dim r As Integer = 0
                Dim g As Integer = 0
                Dim b As Integer = 0
                For dx As Integer = 0 To 2
                    For dy As Integer = 0 To 2
                        bm1_bytes.GetPixel(x + dx - 1, y + dy - 1, new_r, new_g, new_b)
                        r += new_r * m_Filter(dx, dy)
                        g += new_g * m_Filter(dx, dy)
                        b += new_b * m_Filter(dx, dy)
                    Next dy
                Next dx

                r = CInt(127 + r / m_Weight)
                g = CInt(127 + g / m_Weight)
                b = CInt(127 + b / m_Weight)
                If r < 0 Then r = 0
                If g < 0 Then g = 0
                If b < 0 Then b = 0
                If r > 255 Then r = 255
                If g > 255 Then g = 255
                If b > 255 Then b = 255
                bm2_bytes.SetPixel(x, y, r, g, b)
            Next y
        Next x

        ' Unlock the bitmaps.
        bm1_bytes.UnlockBitmap()
        bm2_bytes.UnlockBitmap()
    End Sub
#End Region ' LockBits

#Region "Parallel LockBits"
    ' Process the image in parallel.
    Private Sub btnParallelFilter_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnParallelFilter.Click
        If picOld.Image Is Nothing Then
            MessageBox.Show("You must load an image first", "No Image", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Sub
        End If

        lblParallelTime.Text = ""
        picNew.Image = picOld.Image.Clone()
        Me.Cursor = Cursors.WaitCursor
        Me.Refresh()

        Dim start_time As Date = Now
        DoFilterLockBitsParallel()
        Dim stop_time As Date = Now
        Dim elapsed_time As TimeSpan = stop_time - start_time

        picNew.Refresh()
        lblParallelTime.Text = elapsed_time.TotalSeconds.ToString("0.00") & " sec"
        Me.Cursor = Cursors.Default
    End Sub

    ' Apply the filter in parallel.
    Private Const NUM_TASKS As Integer = 10
    Private m_X0(NUM_TASKS - 1) As Integer
    Private m_X1(NUM_TASKS - 1) As Integer
    Private m_Height As Integer
    Private m_Bm1Bytes As BitmapBytesRGB24
    Private m_Bm2Bytes As BitmapBytesRGB24
    Private Sub DoFilterLockBitsParallel()
        ' Get the bitmaps.
        Dim bm1 As Bitmap = picOld.Image
        Dim bm2 As Bitmap = picNew.Image

        ' Make BitmapBytesRGB24 objects for the bitmaps.
        m_Bm1Bytes = New BitmapBytesRGB24(bm1)
        m_Bm2Bytes = New BitmapBytesRGB24(bm2)

        ' Lock the bitmaps' bytes.
        m_Bm1Bytes.LockBitmap()
        m_Bm2Bytes.LockBitmap()

        ' Calculate the start and end X values for the strips.
        Dim wid As Integer = bm1.Width \ NUM_TASKS
        Dim x0 As Integer = 1
        For task_num As Integer = 0 To NUM_TASKS - 1
            m_X0(task_num) = x0
            m_X1(task_num) = x0 + wid - 1
            x0 += wid
        Next task_num
        m_X1(NUM_TASKS - 1) = bm1.Width - 2
        m_Height = bm1.Height

        ' Process the image in strips.
#Const DO_PARALLEL = True 'False
#If DO_PARALLEL Then
        Parallel.For(0, NUM_TASKS, AddressOf DoFilterStrip)
#Else
        For task_num As Integer = 0 To NUM_TASKS - 1
            DoFilterStrip(task_num)
        Next task_num
#End If

        ' Unlock the bitmaps' bytes.
        m_Bm1Bytes.UnlockBitmap()
        m_Bm2Bytes.UnlockBitmap()
    End Sub

    ' Process pixels for a strip.
    Private Sub DoFilterStrip(ByVal task_num As Integer)
        Dim new_r As Byte = 0
        Dim new_g As Byte = 0
        Dim new_b As Byte = 0
        For x As Integer = m_X0(task_num) To m_X1(task_num)
            For y As Integer = 1 To m_Height - 2
                Dim r As Integer = 0
                Dim g As Integer = 0
                Dim b As Integer = 0
                For dx As Integer = 0 To 2
                    For dy As Integer = 0 To 2
                        m_Bm1Bytes.GetPixel(x + dx - 1, y + dy - 1, new_r, new_g, new_b)
                        r += new_r * m_Filter(dx, dy)
                        g += new_g * m_Filter(dx, dy)
                        b += new_b * m_Filter(dx, dy)
                    Next dy
                Next dx

                r = CInt(127 + r / m_Weight)
                g = CInt(127 + g / m_Weight)
                b = CInt(127 + b / m_Weight)
                If r < 0 Then r = 0
                If g < 0 Then g = 0
                If b < 0 Then b = 0
                If r > 255 Then r = 255
                If g > 255 Then g = 255
                If b > 255 Then b = 255
                m_Bm2Bytes.SetPixel(x, y, r, g, b)
            Next y
        Next x
    End Sub
#End Region ' Parallel LockBits

    ' Exit.
    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        Me.Close()
    End Sub

    ' Save the result image.
    Private Sub mnuFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileSaveAs.Click
        If sfdNew.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Dim file_name As String = sfdNew.FileName
            Dim ext As String = file_name.Substring(file_name.LastIndexOf(".")).ToLower()
            Select Case ext
                Case ".bmp"
                    Dim bm As Bitmap = picNew.Image
                    bm.Save(file_name, ImageFormat.Bmp)
                Case ".jpg", ".jpeg"
                    Dim bm As Bitmap = picNew.Image
                    bm.Save(file_name, ImageFormat.Jpeg)
                Case ".gif"
                    Dim bm As Bitmap = picNew.Image
                    bm.Save(file_name, ImageFormat.Gif)
                Case Else
                    MessageBox.Show("Unknown file extension " & ext & ".", "Unknown File Type", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End Select
        End If
    End Sub
End Class
